/******************************************************************************
	File:			Filter.cpp
	Description:		This class implements quasioptimal filter
	Class:			CFilter
	Author:			AR1795 (06/08/03)							  
	Copyright (C) 2003 AR1795                                                                   
******************************************************************************/

#include "Filter.h"
#include "api.h"

/******************************************************************************
Default constructor
******************************************************************************/
CFilter::CFilter ()
{
  m_wPeriod = DEFAULT_PERIOD;
  m_wThreshold = DEFAULT_THRESHOLD;
}

/******************************************************************************
Constructor to set a filter period and a signal energy threshold
******************************************************************************/
CFilter::CFilter (unsigned short a_wPeriod, unsigned short a_wThreshold)
{
  m_wPeriod = a_wPeriod;
  m_wThreshold = a_wThreshold;
}

/******************************************************************************
Calculates line position in the image
******************************************************************************/
unsigned short CFilter::Calculate(const unsigned char* a_pbyImage, unsigned short a_wSize)
{
	unsigned short wLinePosition = 0xFFFFU;	

	unsigned short wPixel = 0;
	unsigned short wPeriod2 = m_wPeriod/2;
	unsigned short wPeriod4 = m_wPeriod/4;

	unsigned long prevContrast = 0;
	unsigned long currContrast = 0;
	register short prevCosC = 0;
	register short currCosC = 0;
	register short prevSinC = 0;
	register short currSinC = 0;

	for(wPixel = 0; wPixel < wPeriod2; wPixel++)
	{
		prevCosC += a_pbyImage[wPixel];
		prevCosC -= a_pbyImage[wPixel + wPeriod2];
	}
   	for(wPixel = 0; wPixel < wPeriod4; wPixel++)
	{
		prevSinC += a_pbyImage[wPixel];
		prevSinC -= a_pbyImage[wPixel + wPeriod4];
		prevSinC -= a_pbyImage[wPixel + wPeriod2];
		prevSinC += a_pbyImage[wPixel + wPeriod2 + wPeriod4];
	}
	prevContrast = abs(prevCosC) + abs(prevSinC);

	for(wPixel = wPeriod2; wPixel < a_wSize - wPeriod2; wPixel++)
	{
		currCosC = prevCosC - a_pbyImage[wPixel - wPeriod2] 
				 + (a_pbyImage[wPixel]<<1) 
				 - a_pbyImage[wPixel + wPeriod2];
		currSinC = prevSinC - a_pbyImage[wPixel - wPeriod2] 
				 + (a_pbyImage[wPixel - wPeriod4]<<1)  
				 - (a_pbyImage[wPixel + wPeriod4]<<1) 
				 + a_pbyImage[wPixel + wPeriod2];
		currContrast = abs(currCosC) + abs(currSinC);
 		if (prevCosC <= 0 && currCosC > 0 && 
		   ((currContrast + prevContrast)>>1) > m_wThreshold)
		{
				wLinePosition = wPixel;
				break;
					
		}
		prevContrast = currContrast;
		prevCosC = currCosC;
		prevSinC = currSinC;
	}
	return wLinePosition;
}

